import QtQuick 1.1
import com.nokia.meego 1.1 // com.nokia.meego 1.1 includes Screen.allowSwipe (this 1.0 too)
// ** Note: allowSwipe is included from Harmattan PR 1.1 onwards. If your application
// uses this feature, it cannot be used in any device with an older operating system.
// For older OS version, use Binding (target: screenLock) below. Also uncomment in main.cpp
// #include "screenlock.h" plus another line. On MainPage.qml, comment lines with screen.allowSwipe.
// ...and two lines in gkoskeyboard.pro (screenlock.cpp and .h)

import Multitouch 1.0

// import QtMobility.contacts 1.1 // commented 2012-10-09
import QtMobility.feedback 1.1 // for HapticsEffect
import QtMultimediaKit 1.1 // for PlaySound


// import QtMobility.messaging 1.1 // for received messages

import "./settings.js" as Storage
import "./multitouch.js" as GHandler
import "./charactersets.js" as GSets
import "./text.js" as GText



// Spec of Nokia N9: Screen Resolution is 16:9, FWVGA, 854 x 480 pixels


Rectangle { // No window to raise if re-maximized

    id: root
    // x: 0 // TEST
    // y: 0 // TEST
    width: 480
    height: 854
    color: "#000000"
    visible: true


    property bool menuShown: false // Menu is visible on the screen


    Binding {
        target: screenLock
        property: 'locked'
        value: lockEnabled
    }

    Rectangle { // What is this
        id:rootmirror
        anchors.fill: parent
        visible: false

    }

    property bool landscapeOrientation: false

    property variant touchPoints: multitouch.touchPoints
    property variant prevLength: 0 // Number of touchpoints before
    property variant prevChar: "" // previous character typed
    property variant autoChar: "" // previous character typed for autoRepeat
    property variant prevChord: 0 // previous chord typed
    property variant prevOffset: 0 // previous offset of typed character
    property variant presentLength: 0 // Number of touchpoints now

    // Button labels (text):
    property variant labelA: ""
    property variant labelB: ""
    property variant labelC: ""
    property variant labelD: ""
    property variant labelE: ""
    property variant labelF: ""
    property variant labelG: ""
    property variant labelK: ""
    property variant labelO: ""
    property variant labelS: ""
    property variant labelW: ""
    property variant labelTH: ""
    property variant labelBS: ""
    property variant labelSP: ""
    property variant labelAbc: ""
    property variant labelLang: ""
    property variant labelTools: ""
    property variant auxLabel1: "" // depends on language in aux set area (abc)
    property variant auxLabel2: "" // depends on language in aux set area (Abc)

    // Smaller text if text is long
    property variant smallA: 0
    property variant smallS: 0
    property variant smallE: 0
    property variant smallW: 0


    // Graphics on buttons while not text:
    property variant graphA: ""
    property variant graphB: ""
    property variant graphC: ""
    property variant graphD: ""
    property variant graphE: ""
    property variant graphF: ""
    property variant graphG: ""
    property variant graphK: ""
    property variant graphO: ""
    property variant graphS: ""
    property variant graphW: ""
    property variant graphTH: ""
    property variant graphBS: ""
    property variant graphSP: ""
    property variant graphAbc: ""
    property variant graphTools: ""

    // Button states (up/down, for highlighting)
    property variant downA: 0
    property variant downB: 0
    property variant downC: 0
    property variant downD: 0
    property variant downE: 0
    property variant downF: 0
    property variant downG: 0
    property variant downK: 0
    property variant downO: 0
    property variant downS: 0
    property variant downW: 0
    property variant downTH: 0
    property variant downBS: 0
    property variant downSP: 0
    property variant downAbc: 0
    property variant downTools: 0

    // Heplers between keys
    property variant helperA: ""
    property variant helperO: ""
    property variant helperTH: ""
    property variant helperD: ""
    property variant helperG: ""
    property variant helperW: ""
    property bool helpersOn: true

    property variant themeMainButton: "" //  "images/mainButtonPassion.png"
    property variant themeMiddleButton: "" //  "images/mainButtonPassion.png"
    property variant themeBarButton: "" //  "images/mainButtonPassion.png"
    property variant themeLeftBarButton: "" //  "images/mainButtonPassion.png"
    property variant themeTopButton: "" //  "images/mainButtonPassion.png"

    // global parameters
    property variant state: 0 // chord of first finger
    property variant initialState: 0 // for one finger sliding
    property variant newState: 0 // for one finger sliding
    property bool startedSliding: false // sliding started
    property bool middleButtonPressed: false // used for Devanagari consonant detection
    property bool devanagariOn: false // used for Devanagari offset switching
    property variant key: 0 // chord of second finger
    property variant gChord: 0 // total chord value
    property variant gChar: ""
    property bool gReady: false
    property variant linecount: 0
    property bool abcHeld: false
    property bool abcShortcutSelected: false
    property bool toolsShortcutSelected: false

    property variant offset: 0 // shift, nums etc
    // property variant position: 0 // cursor position in the text
    property variant gkosRef: 0 // pointer to the character in the set

    property variant gLanguage: 0 // layout language
    property bool useAuxSets: false // e.g. Korean or English on abc button


    property bool optLayout: true // optimised or alphabetic layout
    property variant opt3: 0 // Experimantal layout (optLayout == true, opt = 0, opt3 =1)
    property bool autoCaps: true // automatic capitalisation
    property bool showHelp: true // little symbols/tips next to the keys
    property bool showSymbs: true // Show/Hide all symbols on keys as well as tips!
    property variant buttonTheme: 0  // 1 = Cyan, 2 = Magenta
    property variant fontSize: 0     // 1/2/3/4 small/normal/large/huge (in use now: 3, 4)
    property variant dataSaved: "" // Indicates whether database exists

    property bool lockEnabledSet: false // swipe disabled in Menu
    property bool lockEnabled: false // swipe disabled in action = !screen.allowSwipe
    property bool numberMode: false // To return to 123 mode from Shift/Symb pressed during 123 mode

    property variant smsText: ""
    property variant smsNumber: ""
    property variant smsPerson: ""
    property variant smsCount: 0 // number of received SMSs
    property variant smsState: "" // Receiced SMS shown/hidden
    property bool smsTimerOn: false // update received SMS display pariodically
    property bool smsWaitOn: false // show: Please wait...

    // property bool gTimerOn: true // to keep track of gTimer on/off (10 ms timer)
    property int waitPeriods: 0 // counting n x 10ms // press longer Arrow up and PgUp
    property int abcPeriods: 0 // counting n x 10ms // press long abc - abcHeld
    property bool autoRepeat: false // abcHeld for a while => autoRepeat true
    property int soundPeriods: 0 // counting n x 10ms // doule click or single
    property int pressPeriods: 0 // counting n x 10ms // prevent double character ouputs
    property int lockPeriods: 0  // counting n x 10ms // enable Swipe after no typing if not completely off
    property int slidePointX: 0  // Present touchPoint values to check if the slide was long enough
    property int slidePointY: 0  //
    property int startPointX: 0  // Saved touchPoint start values to check if the slide will be long enough
    property int startPointY: 0  //
    property bool longSlide: false // sliding long within a key or even outside of the key


    // ============= Settings ======================

    Component.onCompleted: {


        menuShown = false;
        // viewer.showMaximized();

        // for defaults, see settings.js
        dataSaved = "no";
        smsState = "hidden";
        smsTimerOn = false;
        smsWaitOn = false;

        var Language = {
            "English":0, "Esperanto":1, "Estonian":2, "Danish_Norwegian":3, "Finnish":4, "Swedish":5,
            "German":6, "French":7, "Icelandic":8, "Italian":9, "Spanish":10, "Portuguese":11, "Russian":12,
            "Greek":13, "Korean":14, "Ukrainan":15, "Malay": 16 }

        gLanguage = Language.English;

        showSymbs = true;

        themeMainButton = "images/mainButtonCelo.png";
        themeMiddleButton = "images/middleButtonCelo.png";
        themeBarButton = "images/barButtonCelo.png";
        themeLeftBarButton = "images/barButtonCelo.png";
        buttonTheme = 1;

        // Initialize the database
        Storage.initialize();

        // Read all Global settings from database. If not saved yet, get defaults,
        Storage.readSettings();

        lockEnabled = lockEnabledSet; // disable swipe if this option is selected in settings
        // screen.allowSwipe = !lockEnabled;

        edit.text = smsText; // Storage.getSetting("smsText");
        console.log("Stored smsText was: " + smsText);
        linecount = edit.lineCount;

        smsNumber = "1234567890"; // just some content, not used in this version yet (9/2012)

        // put initial labels on buttons
        // ...but this noes not seem to be necessary, signs are shown on buttons anyway
        var gLabel = "";


        auxLabel1 = "aux"; auxLabel2 = "Aux";
        if (autoCaps){offset = 64; labelAbc = "Abc";} else {offset = 0; labelAbc ="abc";}


        if (gLanguage == 14){ // Korean selected
            auxLabel1 ="한글"; auxLabel2 ="한글";
            useAuxSets = true;
            offset = 320;
        }

        edit.cursorPosition = edit.text.length;
        edit.closeSoftwareInputPanel(); // Normal qwerty keypad will not appear
        // See also: closeSoftwareInputPanel, AciveFocusOnPress

        GSets.setLanguage(gLanguage, optLayout);
        labelLang = GSets.languageIndication[gLanguage];

        if (useAuxSets){offset = 320;}

        GSets.setTheme(buttonTheme);

        // Initialise visible status of keyboard
        // Helpers, highlights, chracters and graphics
        GHandler.setKeyboard(0, 0, offset);

        gTimer.running = true;

    }

    Component.onDestruction: {
        // Save all settings
        smsText = edit.text;
        Storage.saveSettings();
    }

// -----
    Timer { // n x 10 ms counters

        id: abcTimer

        interval: 10
        // running: true //
        repeat: true;

        onTriggered: {

            abcPeriods++;
            if (abcPeriods > 100000){
                abcPeriods = 0;
            }

            // Stop timer after 10 s of inactivity to save power
            if ((abcPeriods == 1000) && abcTimer.running) {
                abcTimer.running = false;
                // edit.text = edit.text + " Stop "; // TEST
            }

            if ((abcPeriods == 100) && abcHeld) { // 1 s autoRepeat on
                abcPeriods = 0;
                autoRepeat = true;
            }

            if ((abcPeriods == 8) && autoRepeat && abcHeld) { // 0.08 s repeat period
                abcPeriods = 0;

                // Repeat the saved previous character
                if ((gkosRef < 58)  && !longSlide){ // up to _Del

                    var editText = GText.addCharacter(autoChar, edit.text, offset, gkosRef, edit.cursorPosition);
                    // Show new text
                    edit.text = editText.text;
                    edit.cursorPosition = editText.pos;
                    linecount = edit.lineCount;

                }
            }


        }

    }

// -----

    Timer { // n x 10 ms counters

        id: gTimer

        interval: 10
        // running: true //
        repeat: true;

        onTriggered: {

            waitPeriods++;
            if (waitPeriods > 100000){
                waitPeriods = 0;
            }

            // Stop timer after 10 s of inactivity to save power
            if ((waitPeriods == 1000) && gTimer.running) {
                gTimer.running = false;
                // edit.text = edit.text + " Stop "; // TEST
            }

            pressPeriods++;
            if (pressPeriods > 100000){pressPeriods = 0;}

            soundPeriods++;
            if (soundPeriods > 100000){soundPeriods = 0;}

            if (!lockEnabledSet){
                if (lockPeriods == 100){ // (< was 100 on first distr.) Enable swipe if not typing in n x 10 ms
                    if (lockEnabled){lockEnabled = false;
                    // screen.allowSwipe = true;
                    }
                } else {
                    lockPeriods++;
                    if (!lockEnabled){lockEnabled = true;
                    // screen.allowSwipe = false;
                    }
                }
            }
        }

    }
    // -------------------------------------------------------------------------------



    // Handle the Touch Points in javascript
    // x: 0...730, y: 0...430

    onTouchPointsChanged: {


      //  smsSenderNumber = smsSender.text; // just to update

       // if (menuShown) {return;} // Menu is open, don't read touchpoints on pads


        // what has been pressed? state = first key, key = second key

        if (touchPoints.length == 1) { // ------------- one key pressed -------------------
            presentLength = 1;

            if (prevLength == 0){

                var gGrid = GHandler.gRead(touchPoints[0].x, touchPoints[0].y, 0, 0, 1)

                startPointX = slidePointX; startPointY = slidePointY; // save start points for slide length checking
                if (!gReady) {initialState = gGrid.gState; state = initialState;} // lock state to initial touchpoint
                gChord = initialState;
                gReady = true;
                newState = 0;

                if ((soundPeriods > 4) && (gChord > 0) && (!menuShown)){ // sound only if hit keys
                    gSound();
                }
                soundPeriods = 0;
                waitPeriods = 0;
                gTimer.running = true;  // edit.text = edit.text + " Start "; // TEST !!!!

                // console.log("GKOS pad row/column. Touch1: " + gGrid.gRow1 + "/" + gGrid.gColumn1 +  " Chord=" + gChord)

            }

            // sliding same finger to add second key instead of adding second finger
            if (prevLength == 1){

                gGrid = GHandler.gRead(touchPoints[0].x, touchPoints[0].y, 0, 0, 1)

                var xSlide = Math.abs(slidePointX - startPointX);
                var ySlide = Math.abs(slidePointY - startPointY);

                // Slide must be long enough to be made on purpose (button is 85 x 85)
                if ((xSlide > 40) || (ySlide > 40)){
                    longSlide = true; // needed a global variable to stop autorepeat while sliding from Abc to text area
                } else {longSlide = false;}

                newState = gGrid.gState;

                if (((initialState != newState) && longSlide) || startedSliding) { // not same state as before
                    // Siliding to other buttons
                    startedSliding = true;
                    state = initialState;
                    key = newState;
                    key = GHandler.mirrorIfOnTheSameSide(state, key)
                    gChord = state + key;
                    if (state == 101){gChord += -101;} // slide starts from Tools button = mirror single key
                    // if (state == 100){gChord += -100;} // slide starts from Abc button = mirror single key (needs more work)
                    if (newState == 0){gChord = 0;} // sliding out of pads = no output wanted
                    // sliding starts outside kay areas => left/right + mirrored right/left (e.g. SHIFT)
                    if (initialState == 0){key = GHandler.mirror(newState); state = newState; gChord = key + state;}
                } else {
                    // Sliding within same button only (mirror by sliding left/right within same key area)
                    if ((initialState == newState) && (xSlide > 30)){
                        gChord = GHandler.mirror(newState); state = 0; key = gChord;
                    }

                }


            }

        }



        if (touchPoints.length == 2){ // ------------- two keys pressed -------------------
            presentLength = 2;

            if (prevLength < 2){

                if ((soundPeriods > 5) && (!menuShown)){
                    gSound();
                }

                gReady = true;
                soundPeriods = 0;
                waitPeriods = 0;
                gTimer.running = true;  // edit.text = edit.text + " Start "; // TEST !!!!
            }

            gGrid = GHandler.gRead(touchPoints[0].x, touchPoints[0].y, touchPoints[1].x, touchPoints[1].y, 2)
            state = gGrid.gState; // if second key pressed, no more locked state
            key = gGrid.gKey;
            gChord = state + key;


        }



        if (touchPoints.length > 2){ // ------ more keys not allowed in this application -------
            presentLength = 3;

            return;
        }



        if (touchPoints.length === 0){
            presentLength = 0;

        }

        // console.log(presentLength + " TouchPoints.");

        // ---


        // update visible status of helpers, buttons with highlights, characters and graphics
        GHandler.setKeyboard(state, key, offset);

        // For Devanagari, this info is needed
        if (GHandler.isMiddleButton(state) || GHandler.isMiddleButton(key)){
            middleButtonPressed = true;
        } else middleButtonPressed = false;

        if (gChord == 100){
            abcHeld = true;
            if (autoRepeat){gChord = 0;} // forget abc button's normal function if pressed long
            abcTimer.running = true; //  edit.text = edit.text + " abcStart "; // TEST !!!!
        } // for autorepeat


        // Shorcut functions

        if ((gChord == 0) && (state == 101) && longSlide){ // Tools pressed and then a slide to text area
            // debug
            console.log("Activate Display of received SMS");
            toolsShortcutSelected = true;
        }


        if ((gChord == 0) && (state == 100) && longSlide){ // Abc pressed and then a slide to text area
            // debug
            console.log("Activate Send as SMS");
            abcShortcutSelected = true;
        }



        // keys released?

        if ((presentLength < prevLength) && (presentLength == 0)){ // Thumb(s) up

            if (presentLength == 0){
            abcHeld = false;
            autoRepeat = false;
            abcTimer.running = false; //  edit.text = edit.text + " abcStop "; // TEST !!!!
            }

            if ((initialState == 0) && (newState != 0) && (!menuShown)){ // slided from outside buttons
                gSound();
            }

            // visible button states = all up, no more highliting
            downA = 0; downB = 0; downC = 0; downD = 0; downE = 0; downF = 0;
            downG = 0; downK = 0; downO = 0; downS = 0; downTH = 0; downW = 0;
            downBS = 0; downSP = 0; downAbc = 0; downTools = 0


            // no output if...
            // output was done already earlier, quit
            if (!gReady){return;}



            // both presses on the same side => no chord
            if (GHandler.isOnTheSameSide(state, key)){gChord = 0;}


            // what to output

            if ((gChord < 64) && !menuShown && (pressPeriods > 2)){
                gkosRef = GSets.gChordToRef[gChord]
                gChar = GSets.GKOSChars[gkosRef]
            } else {
                gkosRef = 0; gChar = "";
            }

            // Be more careful with Arrow up/down and Page Up/Dn:
            if ((gChord == 9) || (gChord == 27) || (gChord == 36) || (gChord == 54)){
                if (waitPeriods < 26) {gkosRef = 0; gChar = "";}
            }


            pressPeriods = 0; // reset timer. The timer prevents double characters from a 'single' touch
            if (gkosRef != 0) {lockPeriods = 0;} // reset swipe disable counter if a char is typed
            startedSliding = false;

            // Do something here...

            if ((gLanguage == 14) && (offset > 256) && (gChar !== ""))
            {
                console.log("Char to combiner: " + gChar);
                //gChar = getKoreanCombinedCharacter(c, gChord, Offset)
                gChar = GSets.getKoreanCombinedCharacter(gChar, gChord, offset)
                // console.log("Char from combiner: " + kChar);

            }




            // Show character on screen

            // debug:
            // console.log("QML to JS: Character " + gChar + ", Offset " + offset + ", GKOSRef " + gkosRef )

            // if (gChar != ""){ // ref etc. need also be saved... TODO (now repeat only once, next long press gives nothing)
                autoChar = gChar; // Save previous character for autorepeat before any editing of it
            // }
            prevOffset = offset; // Same reason

            var editText = GText.addCharacter(gChar, edit.text, offset, gkosRef, edit.cursorPosition);

            offset = editText.offset; // offset is global

            // debug:
            console.log(" autoChar [" + autoChar + "] prevChar [" + prevChar + "] Char [" + gChar + "], Offset " + offset )

            if (!devanagariOn && autoCaps && (gChar == " ") && (offset == 0 || offset == 320)){
                if (prevChar == "." || prevChar == "!" || prevChar == "?"){
                    if (offset == 0) {offset = 64;} else {offset = 384;} // Shift on
                }
            } else {
                if(devanagariOn && middleButtonPressed && (offset == 0)){offset = 128;} // Use caps set after consonant
                if(devanagariOn && !middleButtonPressed && (offset == 128)){offset = 0;} // Use lower case set after other keys
            }



            // Show new text
            edit.text = editText.text;
            edit.cursorPosition = editText.pos;
            linecount = edit.lineCount;



            // Put all labels on buttons and helpers back

            // Helpers, highlits, chracters and graphics
            GHandler.setKeyboard(0, 0, offset);


          gReady = false; // No more output on key release this time
          prevChar = gChar; // Save previous character


       // ------------- abc button and Tools menu ------------

            // Tools pressed
            if (gChord == 101){
                if (menuShown) { // Close any menu on second press on Tools button
                    gSound();
                    mainMenu.close();
                    editingMenu.close();
                    settingsMenu.close();
                    languageMenu.close();
                    themeMenu.close();
                    layoutMenu.close();
                    helpMenu.close();
                    // smsMenu.close(); // add when smsMenu is introduced
                    menuShown = false;
                } else {
                menuShown = true; // Uncomment when operation is perfect
                if (soundPeriods < 60){mainMenu.open();} else {editingMenu.open();}
                }
            }

            // abc pressed
            if (gChord == 100){
                // menuShown = true;
                useAuxSets = !useAuxSets;
                if (offset > 256){offset = 0;} else {offset = 320;}
                GHandler.setKeyboard(0, 0, offset);

            }

            // abc+Tools pressed => Show latest received SMS (NEVER COMES TO THIS)
            if (gChord == 201){

                gSound();
                showSmsBox();

            }



       // ----------------------------------------------------
            key = 0; state = 0; // required



            // Shorcuts ------------------------------ // Press a key and slide to text area

            if ((abcShortcutSelected) && (gkosRef ==0)){
                    // Send as SMS
                    // debug
                  console.log("STARTING Send as SMS");
                  abcShortcutSelected = false;

                  gSound();
                  // smshelper.sendsms(smsNumber, edit.text);
                  smscomposer.sendsms(edit.text)
                  menuShown = false;

              }

            if ((toolsShortcutSelected) && (gkosRef == 0)){
                    // Show latest received SMS
                    // debug
                  console.log("STARTING Show received SMS");
                  toolsShortcutSelected = false;

                  gSound();
                  showSmsBox();
                  menuShown = false;

              }
           // ----------------------------------------



     } // end of keys released


        prevLength = touchPoints.length; // save previous number of touchPoints
        prevChord = gChord;








    } // end of on touchpoints changed


    // --------------------------------------------------------------------------------

    // Text area and its orientation

  Rectangle {
        id: text_area_rect
        // anchors.fill: parent

        property variant rwidth: rootmirror.width // works on N9
        property variant rheight: rootmirror.height

        property bool changeOfWidth: false
        property bool changeOfHeight: false
        property bool newOrientation:  false



        onRwidthChanged:  {
            console.log("portrait or landscape?")
            changeOfWidth = true; newOrientation = (changeOfWidth && changeOfHeight)}
        onRheightChanged: {
            changeOfHeight = true; newOrientation  = (changeOfWidth && changeOfHeight)}

        onNewOrientationChanged: {
            if (newOrientation) {
                changeOfWidth = false;
                changeOfHeight = false;

                if (rwidth > rheight) {
                    // landscape
                    landscapeOrientation = true;
                    console.log("landscape")
                } else {
                    // portrait
                    landscapeOrientation = false;
                    console.log("portrait")
                }
            }
        }

        x: landscapeOrientation ? 226 : 0
        width: landscapeOrientation ? 402 : 480
        height: landscapeOrientation ? 480 : (854-480)

        color: (buttonTheme < 3) ? "#1c3341" : "#222222"



        radius: 12
        anchors.top: parent.top
        anchors.topMargin: 0






  // TextEdit does not implement scrolling, following the cursor, or other behaviors
  // specific to a look-and-feel. Have to add flickable scrolling that follows the cursor:

      Flickable {
            id: flick



            // anchors.left: parent.left

            anchors.fill: parent
            contentWidth: edit.paintedWidth
            contentHeight: edit.paintedHeight
            clip: true

            function ensureVisible(r) // Needs work
            {

                if (contentX >= r.x)
                    contentX = r.x;
                else if (contentX+width <= r.x+r.width)
                    contentX = r.x+r.width-width;
                if (contentY >= r.y)
                    contentY = r.y;
                else if (contentY+height <= r.y+r.height)
                    contentY = r.y+r.height-height;
            }



            TextEdit {
                id: edit

                width: flick.width
                height: flick.height

                color: "white"
                focus: true
                cursorVisible: true
                activeFocusOnPress: false
                // text: qsTr("There will be a GKOS keyboard application for Nokia N9.")
                wrapMode: TextEdit.Wrap
                font.pixelSize: (fontSize == 3) ? 32 : 64 // 32=large
                verticalAlignment: TextEdit.AlignVCenter
                onCursorRectangleChanged: flick.ensureVisible(cursorRectangle)

            }

            // --- received messages ----

            // not in use:
//            Rectangle {
//                id: receivedMessageBox
//                width: flick.width
//                height: flick.height
//                color: (buttonTheme < 3) ? "#1c3341" : "#222222"
//                visible: false // true
//            }


            // --- end of received messages ---

        } // end of flickable

   } // end of text_area_rect





// For debugging the touch pointers
//    Repeater {
//        id: repeater
//        visible: false
//        anchors.fill: parent
//        model: touchPoints.length

//        Rectangle {
//            color: (index === 0) ? "blue" : "red"
//            x: touchPoints[index].x - width/2
//            y: touchPoints[index].y - height/2
//            width: 80
//            height: 80
//            radius: 40
//        }
//    }

    // ------------------------------------------------------------------------------

    // Spec of Nokia N9: Screen 16:9 resolution is FWVGA, 854 x 480 pixels

  // Keypads

    Rectangle {
        id: leftPad

        color: "black"

        x: 0;
        width: landscapeOrientation ? 226 : 280
        y: landscapeOrientation ? 0 : 374 // 854 - 480 = 374
        height: landscapeOrientation ? 480 : 480

        // Pad 226x480
        // Button sizes: Main 120x96, Middle 60x96, Bar 290x86, Top 76x86,

        GkosButton { x: 0; y: 0; height: 120; width: 96; label: labelA; fontSize: 46 - 10 * smallA; fontColor: "black"; keyGraph: graphA; keyState: downA ? "down" :"up"; keyImage: themeMainButton}
        GkosButton { x: 0; y: 96+22; height: 60; width: 96; label: labelO; fontSize: 32; fontColor: "white"; keyGraph: graphO; keyState: downO ? "down" :"upMiddle" ; keyImage: themeMiddleButton}
        GkosButton { x: 0; y: 192-12; height: 120; width: 96; label: labelB; fontSize: 46; keyGraph: graphB; keyState: downB ? "down" :"up"; keyImage: themeMainButton; }
        GkosButton { x: 0; y: 288+12; height: 60; width: 96; label: labelS; fontSize: 32 - 10 * smallS; fontColor: "white"; keyGraph: graphS; keyState: downS ? "down" :"upMiddle"; keyImage: themeMiddleButton }
        GkosButton { x: 0; y: 384-22; height: 120; width: 96; label: labelC; fontSize: 46;keyGraph: graphC;  keyState: downC ? "down" :"up" ; keyImage: themeMainButton }
        GkosButton { x: 0+96+30; y: 0+34; height: 34; width: 96-26; label: labelAbc; fontSize: 24; keyGraph: graphAbc; keyState: downAbc ? "down" :"upSmall"; fontColor: "lightgrey"; }
        GkosButton { x: 0+96+22; y: 96+18; height: 96-20; width: 96-10; label: labelTH; fontSize: 32; keyGraph: graphTH; keyState: downTH ? "down" :"upTop"; fontColor: "white"; keyImage: themeTopButton }
        GkosButton { x: 0+96+22; y: 2*96; height: 290; width: 96-10; label: labelBS; fontSize: 32; keyGraph: graphBS; keyState: downBS ? "down" :"upLeftBar"; keyImage: themeLeftBarButton}


        // Language indicator
        Text {
            id: languageIndicator

            color: '#ff666666'
            x: 112
            y: 72
            width: 100
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 20
            text: labelLang

        }


        // Helpers left
        Text {
            id: helpA

            color: "grey"
            x: 100
            y: 0
            width: 12
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 29
            text: helperA
        }
        Text {
            id: helpO

            color: "white"
            x: 100
            y: 110
            width: 12
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 22
            text: helperO
            //    text: qsTr("\u2191")
        }
        Text {
            id: helpTH

            color: "white"
            x: 210
            y: 112
            width: 12
            horizontalAlignment: Text.AlignHCenter
            font.letterSpacing: 2
            font.pixelSize: 22
            text: helperTH
            //    text: qsTr("\u2191")
        }

    } // end of left pad

     Rectangle {
        id: rightPad

        color: "black"

        x: landscapeOrientation ? 854-226 : 480-226
        width:226
        y: landscapeOrientation ? 0 : (854-480)
        height: 480

        // Pad 226x480
        // Button sizes: Main 120x96, Middle 60x96, Bar 290x86, Top 76x86,

        GkosButton { x: 226-96; y: 0; height: 120; width: 96; label: labelD; fontSize: 46; keyGraph: graphD; keyState: downD ? "down" :"up" ; keyImage: themeMainButton}
        GkosButton { x: 226-96; y: 96+22; height: 60; width: 96; label: labelG; fontSize: 32; fontColor: "white"; keyGraph: graphG; keyState: downG ? "down" :"upMiddle" ; keyImage: themeMiddleButton}
        GkosButton { x: 226-96; y: 192-12; height: 120; width: 96; label: labelE; fontSize: 46 - 10 * smallE; keyGraph: graphE; keyState: downE ? "down" :"up" ; keyImage: themeMainButton}
        GkosButton { x: 226-96; y: 288+12; height: 60; width: 96; label: labelK; fontSize: 32; fontColor: "white"; keyGraph: graphK; keyState: downK ? "down" :"upMiddle" ; keyImage: themeMiddleButton}
        GkosButton { x: 226-96; y: 384-22; height: 120; width: 96; label: labelF; fontSize: 46; keyGraph: graphF; keyState: downF ? "down" :"up" ; keyImage: themeMainButton}
        GkosButton { x: 20; y: 0+34; height: 34; width: 96-26; label: labelTools; fontSize: 24; fontColor: "lightgrey"; keyGraph: graphTools; keyState: downTools ? "down" :"upSmall" }
        GkosButton { x: 20; y: 96+18; height: 96-20; width: 96-10; label: labelW; fontSize: 32 - 10 * smallW; fontColor: "white"; keyGraph: graphW; keyState: downW ? "down" :"upTop" ; keyImage: themeTopButton}
        GkosButton { x: 20; y: 2*96; height: 290; width: 96-10; label: labelSP; fontSize: 32; keyGraph: graphSP; keyState: downSP ? "down" :"upBar" ; keyImage: themeBarButton}



        Image {
            id: exitButton
            visible: lockEnabledSet? true : false
            // visible: lockEnabled? true : false // to test
            source: "images/exitButtons.png"
            anchors.top: parent.top
            x: 20
        }



        // Helpers right
        Text {
            id: helpD

            color: "grey"
            x: 114
            y: 0
            width: 12
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 29
            text: helperD
        }
        Text {
            id: helpG

            color: "white"
            x: 112
            y: 112
            width: 12
            horizontalAlignment: Text.AlignHCenter
            font.pixelSize: 22
            text: helperG
            //    text: qsTr("\u2191")
        }
        Text {
            id: helpW

            color: "white"
            x: 6
            y: 112
            font.pixelSize: 22
            font.letterSpacing: 2
            width: 12
            horizontalAlignment: Text.AlignHCenter
            text: helperW
            //    text: qsTr("\u2191")
        }


    } // end of right pad

// =====================================================================================

//     SMSHelper // ADD THIS XXXXXXXXX when SMSHelp works again
//         {
//             id: smshelper

//         }

// =====================================================================================

     Menu {
          id: mainMenu

              platformStyle: MenuStyle {
                  leftMargin: 0
                  rightMargin: 0
                  topMargin: 0
                  bottomMargin: 0
                  leftPadding: 12
                  rightPadding: 12
                  topPadding: 12
                  bottomPadding: 12
                  dim: 0.75
                  fadeInDuration: 450
                  fadeOutDuration: 450
                  fadeInDelay: 50
                  fadeOutDelay: 50
                  // faderBackground: "faderBackground.png"
                  pressDelay: 150
              }

          // visualParent is needed to specify the grayed out area.
             visualParent: text_area_rect



          MenuLayout {

              MenuItem {text: "Send as SMS..."; onClicked: {
                      // smshelper.sendsms("12345678", "Heippa!");
                      gSound();
                      // smshelper.sendsms(smsNumber, edit.text);
                      smscomposer.sendsms(edit.text)
                      menuShown = false;
//                      smsMenu.open();
                  }
              }
              MenuItem {text: "Latest received SMS"; onClicked: {
                      gSound();
                      showSmsBox();
                      // menuShown = false;
                  }
              }
              MenuItem {text: "Edit text..."; onClicked: {
                      gSound();
                      editingMenu.open();
                  }
              }
              MenuItem {text: "<< EXIT keyboard"; onClicked: { // EXIT the whole application
                      gSound();
                      menuShown = false;
                      Qt.quit()
                  }
              }
              MenuItem {text: "Settings..."; onClicked: {
                      gSound();
                      settingsMenu.open();
                  }
              }
              MenuItem {text: "Language..."; onClicked: {
                      gSound();
                      languageMenu.open();
                  }
              }
//              MenuItem {text: "Button themes..."; onClicked: {
//                      gSound();
//                      themeMenu.open();
//                  }
//              }
              MenuItem {text: "Help..."; onClicked: {
                      gSound();
                      helpMenu.open();
                  }
              }

          }
      } // end of main menu

     // --------------------------



    // ------------------------
     Menu {
          id: editingMenu

          platformStyle: MenuStyle {
              leftMargin: 0
              rightMargin: 0
              topMargin: 0
              bottomMargin: 0
              leftPadding: 12
              rightPadding: 12
              topPadding: 12
              bottomPadding: 12
              dim: 0.75
              fadeInDuration: 450
              fadeOutDuration: 450
              fadeInDelay: 50
              fadeOutDelay: 50
              // faderBackground: "faderBackground.png"
              pressDelay: 150
          }

          // visualParent is needed to specify the grayed out area.
          // visualParent: landscapeOrientation ? root : text_area_rect // edit // text_area_rect // root
          visualParent: text_area_rect


          MenuLayout {
              MenuItem {text: "Copy"; onClicked: {
                      gSound();
                      edit.selectAll();
                      edit.copy();
                      menuShown = false;
                  }
              }

              MenuItem {text: "Paste"; onClicked: {
                      gSound();
                      edit.paste();
                      menuShown = false;
                  }
              }

              MenuItem {text: "Delete all text!"; onClicked: {
                      gSound();
                      edit.text = ""; if (offset < 320) {offset = 64;} else {offset = 384;}
                      if (devanagariOn && !useAuxSets){offset = 0;}
                      GHandler.setKeyboard(0, 0, offset);
                      menuShown = false;
                  }
              }

          }
      } // end of editing menu



     // ------------------------
      Menu {
           id: helpMenu

           platformStyle: MenuStyle {
               leftMargin: 0
               rightMargin: 0
               topMargin: 0
               bottomMargin: 0
               leftPadding: 12
               rightPadding: 12
               topPadding: 12
               bottomPadding: 12
               dim: 0.75
               fadeInDuration: 450
               fadeOutDuration: 450
               fadeInDelay: 50
               fadeOutDelay: 50
               // faderBackground: "faderBackground.png"
               pressDelay: 150
           }

           // visualParent is needed to specify the grayed out area.
           // visualParent: landscapeOrientation ? root : text_area_rect // edit // text_area_rect // root
           visualParent: text_area_rect


           MenuLayout {

               MenuItem {text: "How to use"; onClicked: {
                       gSound();
                       messageboxText.font.pixelSize = 24;
                       messageboxText.text = '<b>Chorded Keyboard GKOS</b><br><br>Type with your thumbs.<br>Press one or two keys at<br>the same time (left&#44; right<br>or left &#43; right). Typing with<br>one hand is also possible&#58;<br>Press and slide when<br>needed. - More at:<br>http&#58;&#47;&#47;gkos.com&#47;meego';
                       showBox();
                       menuShown = false;
                   }
               }

               MenuItem {text: "About GKOS"; onClicked: {
                       gSound();
                       messageboxText.font.pixelSize = 32;
                       messageboxText.text = '<b>Chorded Keyboard</b><br><br>GKOS v.0.3.0_pre2<br><br>http&#58;&#47;&#47;gkos.com&#47;meego<br><br>gkos&#64;gkos.com';
                       showBox();
                       menuShown = false;
                   }
               }

           }

       } // end of help menu


    // Message box

      function showBox() { // About GKOS, How to use
            messagebox.state = "shown"
      }

      Rectangle { // Message box
          id: messagebox
          anchors.fill: parent
          color: '#dd000000'
          opacity: 0
          z: 1000

          Rectangle { // box
              width: 420
              height: 460
              radius: 10
              anchors.centerIn: parent
              color: 'lightgrey'

              Text { // Message box info part
                  id: messageboxText
              anchors.centerIn: parent
              // color: "white"
              }

              Rectangle { // Message box OK Button
              width: 80
              height: 40
              radius: 3
              border.color: "#000055"
              border.width: 1
              Image {
                  id: okButton
                  anchors.fill: parent
                  source: themeMiddleButton // (buttonTheme == 1) ? "images/middleButtonBlue.png" : "images/middleButtonCelo.png"
              }

              // color: "#7799ee"
              anchors.horizontalCenter: parent.horizontalCenter
              anchors.bottomMargin: 20
              anchors.bottom: parent.bottom
              Text {
                  anchors.centerIn: parent
                  font.pixelSize: 28
              text: 'OK'
              color: "white"
              }
              MouseArea {
              anchors.fill: parent
              onClicked: {messagebox.state = 'hidden'; gSound();}
              }
          }


      }
      states: [
          State {
          name: "hidden"
          PropertyChanges { target: messagebox; opacity: 0 }
      },
          State {
              name: "shown"
              PropertyChanges { target: messagebox; opacity: 1 }
      }

      ]
      transitions: [
            Transition {
            NumberAnimation { properties: "opacity"; duration: 500 }
          }
      ]
      } // end of message box


      // SMS message box

      function showSmsBox() {
          smsState = "shown";
          smsTimerOn = true;
          smsWaitOn = true;
          // Testing if this works here. It does not:
          // smslist.model.sortOrder = MessageModel.AscendingOrder;
          // smslist.model.sortOrder = MessageModel.DescendingOrder;
          //
      }

      SmsMessage {
          id: smsMessage;
          smsTimerRunning: smsTimerOn;
          smsboxState: smsState;
          // smsWaitVisible: smsWaitOn;
      }


    // ------------------
     Menu {
          id: settingsMenu

          platformStyle: MenuStyle {
              leftMargin: 0
              rightMargin: 0
              topMargin: 0
              bottomMargin: 0
              leftPadding: 12
              rightPadding: 12
              topPadding: 12
              bottomPadding: 12
              dim: 0.75
              fadeInDuration: 450
              fadeOutDuration: 450
              fadeInDelay: 50
              fadeOutDelay: 50
              // faderBackground: "faderBackground.png"
              pressDelay: 150
          }


          // visualParent is needed to specify the grayed out area.
          visualParent: text_area_rect

          MenuLayout {

              // a deeper menu level cause problems if settings are saved each time
              MenuItem {text: "Button themes..."; onClicked: {
                      gSound();
                      themeMenu.open();
                  }
              }

              MenuItem {text: (fontSize == 3)? "Font size: Large/SMALL" : "Font size: LARGE/small"; onClicked: {
                      gSound();
                      if (fontSize == 3){fontSize = 4;} else {fontSize = 3;}
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: lockEnabledSet ? "Screen lock: ON/auto" : "Screen lock: on/AUTO"; onClicked: {
                      gSound();
                      lockEnabledSet = !lockEnabledSet;
                      lockEnabled = lockEnabledSet;
                      // screen.allowSwipe = !lockEnabled;
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: autoCaps? "Automatic Capitals: ON/off" : "Auto Capitals: on/OFF"; onClicked: {
                      gSound();
                      autoCaps = !autoCaps;
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
//              MenuItem {text: optLayout ? "Layout: Alpha/OPTIMIZED" : "Layout: ALPHA/Optimized"; onClicked: {
//                      gSound();
//                    if (optLayout){optLayout = false;} else {optLayout = true;}
//                    GSets.setLanguage(gLanguage, optLayout);
//                    GHandler.setKeyboard(0, 0, offset);
//                    //Storage.saveSettings();
//                    menuShown = false;
//                  }
//              }
               MenuItem {text: showHelp ? "Tips: SHOW/Hide" : "Tips: Show/HIDE"; onClicked: {
                      gSound();
                      if (showHelp){showHelp = false;} else {showHelp = true;}
                      GSets.setLanguage(gLanguage, optLayout);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: !showSymbs ? "Master Mode!!!  ON/off" : "Master Mode!!!  on/OFF"; onClicked: {
                      gSound();
                      if (showSymbs){showSymbs = false;} else {showSymbs = true;}
                      GSets.setLanguage(gLanguage, optLayout);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: "Alphabetic/Optimized layouts..."; onClicked: {
                      gSound();
                      layoutMenu.open();
                  }
              }



          } // end of Menu Layout

     } // end of settings menu


     // --------------------

    Menu {
          id: languageMenu

          platformStyle: MenuStyle {
              leftMargin: 0
              rightMargin: 0
              topMargin: 0
              bottomMargin: 0
              leftPadding: 12
              rightPadding: 12
              topPadding: 12
              bottomPadding: 12
              dim: 0.75
              fadeInDuration: 450
              fadeOutDuration: 450
              fadeInDelay: 50
              fadeOutDelay: 50
              // faderBackground: "faderBackground.png"
              pressDelay: 150
          }


          // visualParent is needed to specify the grayed out area.
          visualParent: text_area_rect

          MenuLayout {
              MenuItem {text: (gLanguage == 0)? "English  \u2605" : "English"; onClicked: {
                      gSound();
                      gLanguage = 0;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 1)? "Esperanto  \u2605" : "Esperanto"; onClicked: {
                      gSound();
                      gLanguage = 1;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 2)? "Estonian  \u2605" : "Estonian"; onClicked: {
                      gSound();
                      gLanguage = 2;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 3)? "Danish/Norewegian  \u2605" : "Danish/Norwegian"; onClicked: {
                      gSound();
                      gLanguage = 3;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 17)? "Devanagari  \u2605" : "Devanagari"; onClicked: {
                      gSound();
                      gLanguage = 17;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 4)? "Finnish  \u2605" : "Finnish"; onClicked: {
                      gSound();
                      gLanguage = 4;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 7)? "French  \u2605" : "French"; onClicked: {
                      gSound();
                      gLanguage = 7;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 6)? "German  \u2605" : "German"; onClicked: {
                      gSound();
                      gLanguage = 6;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 13)? "Greek  \u2605" : "Greek"; onClicked: {
                      gSound();
                      gLanguage = 13;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 8)? "Icelandic  \u2605" : "Icelandic"; onClicked: {
                      gSound();
                      gLanguage = 8;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 9)? "Italian  \u2605" : "Italian"; onClicked: {
                      gSound();
                      gLanguage = 9;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


//              MenuItem {text: "Korean (not ready!)"; onClicked: {
//                      gLanguage = 14;
//                      gSound();
//                      useAuxSets = true;
//                      offset = 320;
//                      auxLabel1 = "한글"; auxLabel2 = "한글";
//                      GSets.setLanguage(gLanguage, optLayout);
//                      GHandler.setKeyboard(0, 0, offset);
//                      //Storage.saveSettings();
//                  }
//              }

              MenuItem {text: (gLanguage == 16)? "Malay  \u2605" : "Malay"; onClicked: {
                      gSound();
                      gLanguage = 16;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }





              MenuItem {text: (gLanguage == 11)? "Portuguese  \u2605" : "Portuguese"; onClicked: {
                      gSound();
                      gLanguage = 11;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 12)? "Russian  \u2605" : "Russian"; onClicked: {
                      gSound();
                      gLanguage = 12;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 10)? "Spanish  \u2605" : "Spanish"; onClicked: {
                      gSound();
                      gLanguage = 10;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 5)? "Swedish  \u2605" : "Swedish"; onClicked: {
                      gSound();
                      gLanguage = 5;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


              MenuItem {text: (gLanguage == 15)? "Ukrainian  \u2605" : "Ukrainian"; onClicked: {
                      gSound();
                      gLanguage = 15;
                      useAuxSets = false;
                      offset = 0;
                      auxLabel1 = "aux"; auxLabel2 = "Aux";
                      GSets.setLanguage(gLanguage, optLayout);
                      labelLang = GSets.languageIndication[gLanguage];
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }


          } // end of MenuLayout (of language menu)

      } // end of languageMenu


     // ------------------------------------

    Menu {
         id: layoutMenu

         platformStyle: MenuStyle {
             leftMargin: 0
             rightMargin: 0
             topMargin: 0
             bottomMargin: 0
             leftPadding: 12
             rightPadding: 12
             topPadding: 12
             bottomPadding: 12
             dim: 0.75
             fadeInDuration: 450
             fadeOutDuration: 450
             fadeInDelay: 50
             fadeOutDelay: 50
             // faderBackground: "faderBackground.png"
             pressDelay: 150
         }


         // visualParent is needed to specify the grayed out area.
         visualParent: text_area_rect

         MenuLayout {
             MenuItem {text: (!optLayout)? "Alphabetic  \u2605" : "Alphabetic"; onClicked: {
                        gSound();
                        optLayout = false; opt3 = 0;
                        GSets.setLanguage(gLanguage, optLayout);
                        // GSets.setTheme(buttonTheme);
                        GHandler.setKeyboard(0, 0, offset);
                        //Storage.saveSettings();
                        menuShown = false;
                 }
             }
             MenuItem {text: ((optLayout) && (opt3 == 0))? "Optimized (Recommended)  \u2605" : "Optimized (Recommended)"; onClicked: {
                         gSound();
                         optLayout = true; opt3 = 0;
                         GSets.setLanguage(gLanguage, optLayout);
                         // GSets.setTheme(buttonTheme);
                         GHandler.setKeyboard(0, 0, offset);
                         //Storage.saveSettings();
                         menuShown = false;
                     }
               }
                 MenuItem {text: ((optLayout) && (opt3 == 1))? "Optimized (Experimental)  \u2605" : "Optimized (Experimental)"; onClicked: {
                         gSound();
                         optLayout = true; opt3 = 1;
                         GSets.setLanguage(gLanguage, optLayout);
                        // GSets.setTheme(buttonTheme);
                         GHandler.setKeyboard(0, 0, offset);
                        //Storage.saveSettings();
                         menuShown = false;
                         }
                     }

       } // end of Menu Layout

    } // end of Layout Menu




     // --------------------
     Menu {
          id: themeMenu

          platformStyle: MenuStyle {
              leftMargin: 0
              rightMargin: 0
              topMargin: 0
              bottomMargin: 0
              leftPadding: 12
              rightPadding: 12
              topPadding: 12
              bottomPadding: 12
              dim: 0.75
              fadeInDuration: 450
              fadeOutDuration: 450
              fadeInDelay: 50
              fadeOutDelay: 50
              // faderBackground: "faderBackground.png"
              pressDelay: 150
          }


          // visualParent is needed to specify the grayed out area.
          visualParent: text_area_rect

          MenuLayout {
              MenuItem {text: (buttonTheme == 1)? "Cloudy  \u2605" : "Cloudy"; onClicked: {
                      gSound();
                      buttonTheme = 1;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: (buttonTheme == 2)? "Neon  \u2605" : "Neon"; onClicked: {
                      gSound();
                      buttonTheme = 2;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: (buttonTheme == 3)? "Warm  \u2605" : "Warm"; onClicked: {
                      gSound();
                      buttonTheme = 3;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: (buttonTheme == 4)? "Healthy  \u2605" : "Healthy"; onClicked: {
                      gSound();
                      buttonTheme = 4;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: (buttonTheme == 5)? "Go Meek  \u2605" : "Go Meek"; onClicked: {
                      gSound();
                      buttonTheme = 5;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }
              MenuItem {text: (buttonTheme == 6)? "School  \u2605" : "School"; onClicked: {
                      gSound();
                      buttonTheme = 6;
                      GSets.setTheme(buttonTheme);
                      GHandler.setKeyboard(0, 0, offset);
                      //Storage.saveSettings();
                      menuShown = false;
                  }
              }

        } // Menu Layout

     } // end of Theme Menu





     // ------------------------

     // At the moment, this menu below is not used as there is no sms recepient given directly
     // NOTE: The SMS recepient line in multitouch.cpp has been commented at the moment !!!!
     // - reason is the recepient field shall be completetly empty

     // - deleted in distribution version -

     // -----------------


     function gSound() {
         rumbleEffect.start();  // plays a rumble effect // only works on device (not on simulator)
         gkosKeySound.play();   // sound file for key press click
     }

     HapticsEffect {
          id: rumbleEffect
          attackIntensity: 0.0
          attackTime: 250
          intensity: 1.0
          duration: 100
          fadeTime: 250
          fadeIntensity: 0.0
      }


      SoundEffect {
          id: gkosKeySound
          volume: 0.30
          source: "pvc_bubble.wav"
      }



} // end of root rectangle
