import QtQuick 1.1
import QtMobility.messaging 1.1 // for received messages
// import QtMobility.contacts 1.1 // Commented 2012-10-09
import QtMobility.systeminfo 1.1 // for ScreenSaver functions

// SMS message box

import "./multitouch.js" as GHandler




  Rectangle { // Message box

      id: smsbox

      property alias smsboxState: smsbox.state // use the latter names in this file (smsState "shown"/"hidden")
      property alias smsTimerRunning: smsTimer.running // (smsTimerOn true/false)

      // property alias smsWaitVisible: a10
      property bool waitVisible: true
      property bool landscapeOrientationPrev: false
      property int sPeriods: 0
      property variant triggedCount: 4
      property variant updateInterval: 1000

      anchors.fill: parent
      color: '#dd000000'
      opacity: 0
      z: 1000


      ScreenSaver { id: screenSaver
              Component.onCompleted: {screenSaver.setScreenSaverDelayed(false) }
          }

      // ----------


      Item {
          id: clock
          width: 200; height: 230

          //property alias city: cityLabel.text
          property int hours
          property int minutes
          property int seconds
          property real shift
          property bool night: false


          function timeChanged() {
              // debug:
              // console.log("Clock tick ")

              var clockDate = new Date;
              hours = shift ? clockDate.getUTCHours() + Math.floor(clock.shift) : clockDate.getHours()
              night = ( hours < 8 || hours > 21 )
              minutes = shift ? clockDate.getUTCMinutes() + ((clock.shift % 1) * 60) : clockDate.getMinutes()
              // seconds = clockDate.getUTCSeconds();
          }

          Timer {
              id: clockTimer
              triggeredOnStart: true
              interval: 2000; // 2 s intervals
              running: false; repeat: true;
              onTriggered: {
                   sPeriods = sPeriods + 1;

                  // Correct clock position sooner after turning device
                  if (landscapeOrientation === !landscapeOrientationPrev) {
                        landscapeOrientationPrev = landscapeOrientation;
                        blackScreenClock.anchors.topMargin = landscapeOrientation ? GHandler.randomNumber(0, 260) : GHandler.randomNumber(0, 660); // max 260 : 660
                        blackScreenClock.anchors.leftMargin = landscapeOrientation ? GHandler.randomNumber(0, 560) : GHandler.randomNumber(0, 160); // max 560 : 160

                  }
                  if (sPeriods >= 30){
                  // if (sPeriods >= 2){ // test
                    screenSaver.setScreenSaverDelayed(true);
                    sPeriods = 0;
                    // debug:
                    // console.log("Screensaver delayed ")
                    // Move might clock on screen randomly:
                                      // Screen is 854 x 480 pixels
                    blackScreenClock.anchors.topMargin = landscapeOrientation ? GHandler.randomNumber(0, 260) : GHandler.randomNumber(0, 660); // max 260 : 660
                    blackScreenClock.anchors.leftMargin = landscapeOrientation ? GHandler.randomNumber(0, 560) : GHandler.randomNumber(0, 160); // max 560 : 160
                  }
                  clock.timeChanged();

              }

          }
    }


      Rectangle { // box
          id: smscountbox

          anchors.fill: parent
          radius: 20
          border.color: "#444444"
          border.width: 2
          anchors.centerIn: parent
          color: (buttonTheme < 3) ? "#010304" : "#000000"


          Timer { // n x 500 ms counters

              id: smsTimer
              // interval: waitVisible ? 3000 : 2000 // was: 5000 : 2000
              interval: updateInterval
              // running: true;
              repeat: true;
              triggeredOnStart: true; // immediate trigger first

              onTriggered: {

                  triggedCount = triggedCount - 1;
                  if (triggedCount === 0){
                      waitVisible = false;
                      triggedCount = 1;
                      updateInterval = 1000;
                  }
                  // debug:
                  console.log(" >>>>>>> smsTimer triggered <<<<<<<<")
                  smslist.model.limit = 2; // to rebuild the model
                  smslist.model.limit = 1;
  //                smslist.model.sortOrder = MessageModel.AscendingOrder;
  //                smslist.model.sortOrder = MessageModel.DescendingOrder;
                  screenSaver.setScreenSaverDelayed(true); // This just delays the screensaver, must be repeated
              }
          }



      ListView {
              id: smslist
              anchors.fill: parent
              model: MessageModel {
                  id: smslistModel
                  sortBy: MessageModel.ReceptionTimestamp // in use
                  // sortBy: MessageModel.Timestamp
                  sortOrder: MessageModel.DescendingOrder
                  limit: 1 // only show the latest one


                  // onModelReset: { // This works and can be used instead of timer
                  // onMessageAdded: { // This does not work, so timer is used
                  // onLimitChanged: { //
                  //    smslist.model.sortOrder = MessageModel.AscendingOrder;
                  //    smslist.model.sortOrder = MessageModel.DescendingOrder;
                  // }


                  //  MessageModel.count:

                  filter: MessageIntersectionFilter {
                          id: filters
//                           MessageFilter {
//                               id: filter1
//                               type: MessageFilter.Size
//                               value: 1024
//                               comparator: MessageFilter.LessThan
//                           }
                           MessageFilter {
                               id: filter2
                               type: MessageFilter.Type
                               value: MessageFilter.Sms
                               comparator: MessageFilter.Equal
                           }
                           MessageFilter { // This filter reduces search time from 15 sec to 6 sec !
                               id: filter3
                               negated: true
                               type: MessageFilter.Sender
                               value: "/" // This is hack 1 to filter out sent SMSs
                               comparator: MessageFilter.Includes
                           }


                      } // end of 'filter:'

              }

           delegate: Item {
                  id: wrapper
                  // height: 32;
                  height: smslist.height
                  width: smslist.width

             Flickable {
                      id: flickSMS

                      anchors.fill: parent
                      contentWidth: smslist.width  // 2012-09-29
                      contentHeight: smsSender.paintedHeight + smsBody.paintedHeight + smsDate.paintedHeight
                      // clip: true


                  Text {
                      id: smsWaiting
                      font.pixelSize: (fontSize == 3) ? (32 - 10) : (56 - 16)
                      text: "Wait 10 sec for refresh!";
                      visible: waitVisible
                      color: "grey";
                      x: 3; width: parent.width;
                      wrapMode: Text.Wrap
                  }


                  Text {
                      id: smsSender
                      font.pixelSize: (fontSize == 3) ? 32 : 56
                      text: sender; // waitVisible ? "Please wait..." : sender;
                      visible: true; // !waitVisible
                      color: "yellow";
                      x: 3; width: parent.width;
                      wrapMode: Text.Wrap
                      anchors.top: waitVisible ? smsWaiting.bottom : parent.top;
                      anchors.topMargin: 3;
                      // anchors.top: smssReceived.bottom; anchors.topMargin: 3
                      // elide: Text.ElideRight
                  }


                  Text {
                      id: smsBody
                      text: body; color: "white";
                      visible: true; // !waitVisible
                      width: parent.width;
                      font.pixelSize: (fontSize == 3) ? 32 : 64
                      wrapMode: Text.Wrap
                      anchors.top: smsSender.bottom; anchors.topMargin: 3
                  }


                  Text {
                      id: smsDate
                      text: date; color: "gray";
                      visible: true; // !waitVisible
                      width: parent.width;
                      font.pixelSize: (fontSize == 3) ? 32-10 : 64-30
                      wrapMode: Text.Wrap
                      // anchors.right: parent.right
                      anchors.top: smsBody.bottom; anchors.topMargin: 3
                  }

// ---------------

                  // BlackScreen click area inside flickable

                MouseArea { // parent = Flickable

                    //  screen 854 x 480 pixels
                    width: parent.width  // width is whole screen
                    height: landscapeOrientation ? 480 - 90 : 854 - 90
                    anchors.top: parent.top

                    onMousePositionChanged: {
                        // This will allow time for scrolling (polling period longer)
                        // waitVisible = true;
                        triggedCount = 2;
                        updateInterval = 8000;
                    }

                    onDoubleClicked: {
                          gSound();
                          blackScreen.visible = !blackScreen.visible
                          smsTimerOn = !smsTimerOn;
                          // waitVisible = true;
                          clockTimer.running = !smsTimerOn;
                          menuShown = false;
                          mainMenu.close();
                    }

                } // end of mousearea

// ----------------

              } // end of flickable

           } // end of delegate

       } // end of ListView



     Rectangle { // smsbox OK Button
          width: 80
          height: 40
          radius: 3
          border.color: "#888888"
          border.width: 1
          Image {
              id: okSmsButton
              anchors.fill: parent
              source: (buttonTheme < 3) ? "images/middleButtonBlue.png" : "images/middleButtonPassion.png"
          }

          anchors.horizontalCenter: parent.horizontalCenter
          anchors.bottomMargin: 20
          anchors.bottom: parent.bottom
          Text {
              anchors.centerIn: parent
              font.pixelSize: 28
          text: 'OK'
          color: "white"
          }

      }

            // OK button click area
          MouseArea { // parent = The whole message box

              width: 160  // Area is some larger than the small button
              height: 80
              anchors.horizontalCenter: parent.horizontalCenter
              anchors.bottomMargin: 10
              anchors.bottom: parent.bottom

              onClicked: {
              //smsbox.state = 'hidden';
                    gSound();
                    smsState = "hidden";
                    menuShown = false;
                    smsTimerOn = false;
                    clockTimer.running = false;
                    waitVisible = true;
                    triggedCount = 10;
                    updateInterval = 1000;
                    screenSaver.setScreenSaverDelayed(false);

              }

          }


         // BlackScreen visible area
        Rectangle {
            id: blackScreen
            color: "black"
            opacity: 1
            visible: false
            anchors.fill: parent

            MouseArea {

                anchors.fill: parent

                onDoubleClicked: {
                      gSound();
                      blackScreen.visible = !blackScreen.visible
                      smsTimerOn = !smsTimerOn;
                      // waitVisible = true;
                      clockTimer.running = !smsTimerOn;
                      menuShown = false;
                      mainMenu.close();
                }

            }

            Rectangle {
                id: blackScreenClock
                anchors.top: parent.top
                anchors.left: parent.left
                // height: landscapeOrientation ? 480 - 90 : 854 - 90
                // anchors.top: parent.top

                Text {
                    // color: "#222222"
                    color: clock.night ? "#444444" : "darkgrey"
                    id: blackScreenText
                    font.pixelSize: 120
                    text: (clock.minutes < 10) ? clock.hours + ":0" + clock.minutes : clock.hours + ":" + clock.minutes
                    // text: qsTr("18:34")
                }
                Text {
                    color: clock.night ? "#444444" : "darkgrey"
                    // color: "#222222"
                    id: blackScreenText2
                    anchors.top:blackScreenText.bottom
                    font.pixelSize: 24
                    text: qsTr("  Double tap to exit clock")
                }
             }
         } // Black screen visible area ends



  }
  states: [
      State {
      name: "hidden"
      PropertyChanges { target: smsbox; opacity: 0 }
  },
      State {
          name: "shown"
          PropertyChanges { target: smsbox; opacity: 1 }
  }

  ]
  transitions: [
        Transition {
        NumberAnimation { properties: "opacity"; duration: 500 }
      }
  ]
  }




// ------------------

