//settings.js

// Create a short helper function to get the database connection

function getDatabase() {
     return openDatabaseSync("GkosKeyboardData", "1.0", "StorageDatabase", 100000);
}


// Initialize the tables needed if they haven't been created yet

function initialize() {
    var db = getDatabase();
    db.transaction(
        function(tx) {
            // Create the settings table if it doesn't already exist
            // If the table exists, this is skipped
            tx.executeSql('CREATE TABLE IF NOT EXISTS settings(setting TEXT UNIQUE, value TEXT)');
          });
}


// Save several global settings into the database

function saveSettings(){

    dataSaved = "yes";
    setSetting("dataSaved", dataSaved);
    setSetting("gLanguage", gLanguage);
    setSetting("useAuxSets", useAuxSets);
    setSetting("optLayout", optLayout);
    setSetting("showHelp", showHelp);
    setSetting("buttonTheme", buttonTheme);
    setSetting("fontSize", fontSize);
    setSetting("smsText", smsText);
    setSetting("smsNumber", smsNumber);
    setSetting("autoCaps", autoCaps);
    setSetting("lockEnabledSet", lockEnabledSet);
    setSetting("showSymbs", showSymbs);
    setSetting("opt3", opt3);
}

function readSettings(){

    dataSaved = getSetting("dataSaved");
    gLanguage = parseInt(getSetting("gLanguage"));
    useAuxSets = getSetting("useAuxSets")
    optLayout = getSetting("optLayout");
    showHelp = getSetting("showHelp");
    buttonTheme = parseInt(getSetting("buttonTheme"));
    fontSize = parseInt(getSetting("fontSize"));
    smsText = getSetting("smsText");
    smsNumber = getSetting("smsNumber");
    autoCaps = getSetting("autoCaps");
    lockEnabledSet = getSetting("lockEnabledSet")
    showSymbs = getSetting("showSymbs");
    opt3 = getSetting("opt3");

    if (dataSaved !== "yes"){
        setDefaults();
        saveSettings();
        // console.log("Saving defaults");
    }

}

function setDefaults(){

    gLanguage = 0;
    useAuxSets = false;
    optLayout = true;
    showHelp = true;
    buttonTheme = 1;
    fontSize = 3;
    smsText = "Use your thumbs to type. \n\nOne or two at the same time.";
    smsNumber = "1234";
    autoCaps = true;
    lockEnabledSet = false;
    showSymbs = true;
    opt3 = 0;

}

// This function is used to write one setting into the database

function setSetting(setting, value) {
   // setting: string representing the setting name (eg: “username”)
   // value: string representing the value of the setting (eg: “myUsername”)
   var db = getDatabase();
   var res = "";
   db.transaction(function(tx) {
        var rs = tx.executeSql('INSERT OR REPLACE INTO settings VALUES (?,?);', [setting,value]);
              //console.log(rs.rowsAffected)
              if (rs.rowsAffected > 0) {
                res = "OK";
              } else {
                res = "Error";
              }
        }
  );
  // Returns “OK” if it was successful, or “Error” if it wasn't
  return res;
}


// This function is used to retrieve one setting from the database

function getSetting(setting) {
   var db = getDatabase();
   var res="";
   db.transaction(function(tx) {
     var rs = tx.executeSql('SELECT value FROM settings WHERE setting=?;', [setting]);
     if (rs.rows.length > 0) {
          res = rs.rows.item(0).value;
     } else {
         res = "Unknown";
     }
  })
  // The function returns “Unknown” if the setting was not found in the database
  // For more advanced projects, this should probably be handled through error codes
  return res
}
